--[[
	EN: 'VariableCapacityTrailerShovel' specialization for trailer without the 'VariableBody' specialization and all shovels.
	
	DE: 'VariableCapacityTrailerShovel' Spezialisierung für Anhänger ohne der 'VariableBody' Spezialisierung und allen Schaufeln.
	
	Autor: Ifko[nator]
	Datum: 15.09.2014
	Version: 3.5
	
	Edit am: 30.10.2014
	Edit: LS 15 Ready
	Edit by: Ifko[nator]
	
	Alternativer XML Eintrag:
	
<!--Variable Kapazität Anfang-->
	<variableCapacity addMin="0.01" addMax="0.08" removeMin="0.005" removeMax="0.02" hudPosX="0.3" hudPosY="0.025"/> 
	<!-- Die hier eingetragen Zahlen werden mit der Zahl aus dem Abschnitt 'capacity' multipliziert.
	Hat man nun eine Kapazität von '2000' würden die Werte wie folgt aussehen:
	addMin = '20' addMax = '160' removeMin = '10' removeMax = '40'
	Den: 2000*0.01 = '20', 2000*0.08 = '160', 2000*0.005 = '10' und 2000*0.02 = '40'
	Mit den Werten bei 'hudPosX' und 'hudPosY' kann man die Postion des Huds verändern. Muss man aber nicht angeben.
Variable Kapazität Ende-->
]]

VariableCapacityTrailerShovel = {}

local variableCapacityTrailer_directory = g_currentModDirectory;

function VariableCapacityTrailerShovelprerequisitesPresent(specializations)
	if self.capacity > 0 then
		return SpecializationUtil.hasSpecialization(Fillable, specializations);
	else
		return nil;
	end;
end;

function VariableCapacityTrailerShovel:load(xmlFile)
	if self.capacity > 0 then
		self.capacityStart = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.capacity"), 0);
		
		self.addCapacityMinXml = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.variableCapacity#addMin"), 0.01);
		self.addCapacityMaxXml = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.variableCapacity#addMax"), 0.08);
		
		self.removeCapacityMinXml = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.variableCapacity#removeMin"), 0.005);
		self.removeCapacityMaxXml = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.variableCapacity#removeMax"), 0.02);
		
		self.addCapacityMin = self.capacityStart * self.addCapacityMinXml;
		self.addCapacityMaxStart = self.capacityStart * self.addCapacityMaxXml;
		self.addCapacityMax = self.addCapacityMaxStart;
		
		self.removeCapacityMin = self.capacityStart * self.removeCapacityMinXml;
		self.removeCapacityMax = self.capacityStart * self.removeCapacityMaxXml;
		
		self.addCapacityTime = false;
		self.newAddCapacity = 0;
		self.newRemoveCapacity = 0;
		self.capacityTime = 0;
		self.setNewCapacityTime = 0;
		
		self.setNewCapacity = false;
		self.allowAddCapacity = true;
		self.showHud = false;
		
		self.VariableCapacityTrailerHudPosX = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.variableCapacity#hudPosX"), 0.3);
		self.VariableCapacityTrailerHudPosY = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.variableCapacity#hudPosY"), 0.025);
		self.VariableCapacityTrailerHudWidth = 0.42;
		self.VariableCapacityTrailerHudHeight = 0.23;
		
		self.VariableCapacityTrailerOverlayFilename = 
			Utils.getFilename("GUI/VariableCapacity_Hud.png", 
			variableCapacityTrailer_directory
		);
		self.VariableCapacityTrailerOverlay = 
			Overlay:new("VariableCapacityTrailerOverlay", 
			self.VariableCapacityTrailerOverlayFilename, 
			self.VariableCapacityTrailerHudPosX, 
			self.VariableCapacityTrailerHudPosY, 
			self.VariableCapacityTrailerHudWidth, 
			self.VariableCapacityTrailerHudHeight
		);
	end;
end;

function VariableCapacityTrailerShovel:delete()
end;

function VariableCapacityTrailerShovel:mouseEvent(posX, posY, isDown, isUp, button)
end;

function VariableCapacityTrailerShovel:keyEvent(unicode, sym, modifier, isDown)
end;

function VariableCapacityTrailerShovel:update(dt)
	if self.capacity > 0 then
		if self:getIsActive() and self.isClient and self.isSelected then
			if InputBinding.hasEvent(InputBinding.ADD_CAPACITY_BUTTON) then
				if self.allowAddCapacity then
					self.allowAddCapacity = false;
				else
					self.allowAddCapacity = true;
					if self.capacityTime < 2 then
						self.capacity = self.capacityStart + self.newAddCapacity;
					else
						self.capacity = self.capacityStart - self.newRemoveCapacity;
					end;
				end;
			end;
			if InputBinding.hasEvent(InputBinding.SHOW_HUD_BUTTON) then
				self.showHud = not self.showHud;
			end;
			if (self.fillLevel >= (self.capacity * 0.01) and self.fillLevel < (self.capacity * 0.95)) and self.allowAddCapacity and self.setNewCapacityTime <= 50 then
				if InputBinding.hasEvent(InputBinding.SET_NEW_CAPACITY_BUTTON) then
					self.setNewCapacity = not self.setNewCapacity;
				end;
			else
				self.setNewCapacity = false;
			end;
		end;
	end;
end;

function VariableCapacityTrailerShovel:updateTick(dt)
	if self.capacity > 0 then
		if self.allowAddCapacity then
			if self.fillLevel < (self.capacity * 0.01) or self.setNewCapacity then
				self.addCapacityTime = true;
				if self.currentFillType == Fillable.FILLTYPE_WHEAT 
					or self.currentFillType == Fillable.FILLTYPE_BARLEY
					or self.currentFillType == Fillable.FILLTYPE_OAT
					or self.currentFillType == Fillable.FILLTYPE_RYE
					or self.currentFillType == Fillable.FILLTYPE_WEED
					or self.currentFillType == Fillable.FILLTYPE_RAPE
					or self.currentFillType == Fillable.FILLTYPE_MAIZE
					or self.currentFillType == Fillable.FILLTYPE_FORAGE
					or self.currentFillType == Fillable.FILLTYPE_SUNFLOWER
				then	
					self.addCapacityMax = self.addCapacityMaxStart * 1.1;
				elseif self.currentFillType == Fillable.FILLTYPE_MANURE
					or self.currentFillType == Fillable.FILLTYPE_WHEAT_WINDROW
					or self.currentFillType == Fillable.FILLTYPE_BARLEY_WINDROW
					or self.currentFillType == Fillable.FILLTYPE_GRASS_WINDROW
					or self.currentFillType == Fillable.FILLTYPE_DRYGRASS_WINDROW
					or self.currentFillType == Fillable.FILLTYPE_FUEL
					or self.currentFillType == Fillable.FILLTYPE_MILK
					or self.currentFillType == Fillable.FILLTYPE_SILAGE
				then
					self.addCapacityMax = self.addCapacityMaxStart * 1.3;
				elseif self.currentFillType == Fillable.FILLTYPE_POTATO
					or self.currentFillType == Fillable.FILLTYPE_SUGARBEET
				then
					self.addCapacityMax = self.addCapacityMaxStart * 0.3;
				elseif self.currentFillType == Fillable.FILLTYPE_CHAFF
					or self.currentFillType == Fillable.FILLTYPE_WOODCHIPS
				then
					self.addCapacityMax = self.addCapacityMaxStart * 1.6;
				else
					self.addCapacityMax = self.addCapacityMaxStart;
				end;
				self.newAddCapacity = math.random(self.addCapacityMin, self.addCapacityMax);
				self.newRemoveCapacity = math.random(self.removeCapacityMin, self.removeCapacityMax);
				if self.capacityTime < 2 then
					self.capacity = self.capacityStart - self.newRemoveCapacity;
				else
					self.capacity = self.capacityStart + self.newAddCapacity;
					self.capacityTime = 0;
				end;
			else
				self.addCapacityTime = false;
			end;
			if self.addCapacityTime then
				self.capacityTime = self.capacityTime + 1;
			end;
			if self.setNewCapacity then
				self.setNewCapacityTime = self.setNewCapacityTime + 1;
			else
				self.setNewCapacityTime = 0;
			end;
		else
			self.capacity = self.capacityStart;
			if self.fillLevel > self.capacityStart then
				self.fillLevel = self.capacityStart;
			end;
		end;
	end;
end;

function VariableCapacityTrailerShovel:getSaveAttributesAndNodes(nodeIdent)
	if self.capacity > 0 then
		local showHud = "false";
		local setNewCapacity = "false";
		
		if self.showHud then
			showHud = "true";
		else
			showHud = "false";
		end;
		
		if self.setNewCapacity then
			setNewCapacity = "true";
		else
			setNewCapacity = "false"
		end;
		
		local attributes = 
			'newCapacity="' .. self.capacity .. 
			'" newAddCapacity="' .. self.newAddCapacity .. 
			'" newRemoveCapacity="' .. self.newRemoveCapacity .. 
			'" capacityTime="' .. self.capacityTime .. 
			'" showHud="' .. showHud .. 
			'" setNewCapacity="' .. setNewCapacity .. 
			'" setNewCapacityTime="' .. self.setNewCapacityTime ..
			'"';
		
		return attributes;
	else
		return nil;
	end;
end;

function VariableCapacityTrailerShovel:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if self.capacity > 0 then
		local newCapacity = getXMLInt(xmlFile, key .. "#newCapacity");
		local newFillLevel = getXMLInt(xmlFile, key .. "#fillLevel");
		local newAddCapacity = getXMLInt(xmlFile, key .. "#newAddCapacity");
		local newRemoveCapacity = getXMLInt(xmlFile, key .. "#newRemoveCapacity");
		local capacityTime = getXMLInt(xmlFile, key .. "#capacityTime");
		local showHud = getXMLBool(xmlFile, key .. "#showHud");
		local setNewCapacity = getXMLBool(xmlFile, key .. "#setNewCapacity");
		local setNewCapacityTime = getXMLInt(xmlFile, key .. "#setNewCapacityTime");
		
		if newCapacity ~= nil then
			self.capacity = newCapacity;	
		end;
		
		if newFillLevel ~= nil then
			self.fillLevel = newFillLevel;
		end;
		
		if newAddCapacity ~= nil then
			self.newAddCapacity = newAddCapacity;
		end;
		
		if newRemoveCapacity ~= nil then
			self.newRemoveCapacity = newRemoveCapacity;
		end;
		
		if capacityTime ~= nil then
			self.capacityTime = capacityTime;
		end;
		
		if showHud ~= nil then
			self.showHud = showHud;
		end;
		
		if setNewCapacity ~= nil then
			self.setNewCapacity = setNewCapacity;
		end;
		
		if setNewCapacityTime ~= nil then
			self.setNewCapacityTime = setNewCapacityTime;
		end;
		
		return BaseMission.VEHICLE_LOAD_OK;
	else
		return nil;
	end;
end;

function VariableCapacityTrailerShovel:draw()
	if self.capacity > 0 then
		local FillLevel = string.format(math.floor(self.fillLevel));
		local FillLevelString = string.format("%d [l]", FillLevel);
		
		local Capacity = string.format(math.floor(self.capacity));
		local CapacityStart = string.format(math.floor(self.capacityStart));
		local CapacityStartString = string.format("%d [l]", CapacityStart);
		local CapacityString = string.format("%d [l]", Capacity);
		local CapacityStringUnknown = (Trailer.UNKNOWN);
		
		local NewAddCapacity = string.format(math.floor(self.newAddCapacity));
		local NewAddCapacityString = string.format("%d [l]", NewAddCapacity);
		
		local AddCapacityMin = string.format(math.floor(self.addCapacityMin));
		local AddCapacityMinString = string.format("%d [l]", AddCapacityMin);
		local AddCapacityMax = string.format(math.floor(self.addCapacityMax));
		local AddCapacityMaxString = string.format("%d [l]", AddCapacityMax);
		
		local NewRemoveCapacity = string.format(math.floor(self.newRemoveCapacity));
		local NewRemoveCapacityString = string.format("%d [l]", NewRemoveCapacity);
	
		local RemoveCapacityMin = string.format(math.floor(self.removeCapacityMin));
		local RemoveCapacityMinString = string.format("%d [l]", RemoveCapacityMin);
		local RemoveCapacityMax = string.format(math.floor(self.removeCapacityMax));
		local RemoveCapacityMaxString = string.format("%d [l]", RemoveCapacityMax);
		
		if self.allowAddCapacity then
			g_currentMission:addHelpButtonText(Trailer.NOT_ALLOW_ADD_CAPACITY, InputBinding.ADD_CAPACITY_BUTTON);
			if self.fillLevel >= (self.capacity * 0.01) and self.fillLevel < (self.capacity * 0.95) then
				if self.setNewCapacity then
					g_currentMission:addHelpButtonText(Trailer.STOP_SET_NEW_CAPACITY, InputBinding.SET_NEW_CAPACITY_BUTTON);
				else
					g_currentMission:addHelpButtonText(Trailer.SET_NEW_CAPACITY, InputBinding.SET_NEW_CAPACITY_BUTTON);
				end;
			end;
		else
			g_currentMission:addHelpButtonText(Trailer.ALLOW_ADD_CAPACITY, InputBinding.ADD_CAPACITY_BUTTON);
		end;
		if self.showHud then
			self.VariableCapacityTrailerOverlay:render();
			
			setTextBold(true);
			setTextAlignment(RenderText.ALIGN_CENTER);
			setTextColor(1, 1, 1, 1);
			
			renderText(self.VariableCapacityTrailerHudPosX + 0.205, self.VariableCapacityTrailerHudPosY + 0.185, 0.022, (Trailer.VERSION));
			
			setTextAlignment(RenderText.ALIGN_LEFT);
			
			renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.15, 0.02, (Trailer.FILL_LEVEL));
			renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.13, 0.02, (Trailer.CAPACITY));
			
			if not self.setNewCapacity then
				if self.fillLevel > (self.capacity * 0.01) then
					if self.capacityTime < 2 then
						renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (Trailer.CAPACITY_ADDED));
						if self.newAddCapacity > 0 then
							if self.allowAddCapacity then
								renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.09, 0.02, (Trailer.ADD_CAPACITY_ACTIVE));
							else
								renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.09, 0.02, (Trailer.ADD_CAPACITY_NOT_ACTIVE));
							end;
						else
							renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.09, 0.02, (Trailer.ADD_CAPACITY_NOT_ACTIVE));
						end;
						renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (Trailer.ADD_CAPACITY_MIN));
						renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (Trailer.ADD_CAPACITY_MAX));
					else
						renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (Trailer.CAPACITY_REMOVED));
						if self.newRemoveCapacity > 0 then
							if self.allowAddCapacity then
								renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.09, 0.02, (Trailer.REMOVE_CAPACITY_ACTIVE));
							else
								renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.09, 0.02, (Trailer.REMOVE_CAPACITY_NOT_ACTIVE));
							end;
						else
							renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.09, 0.02, (Trailer.REMOVE_CAPACITY_NOT_ACTIVE));
						end;
						renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (Trailer.REMOVE_CAPACITY_MIN));
						renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (Trailer.REMOVE_CAPACITY_MAX));
					end;
				else
					renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (Trailer.CAPACITY_UNKNOWN));
					renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (Trailer.MIN_UNKNOWN));
					renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (Trailer.MAX_UNKNOWN));
				end;
			else
				renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (Trailer.CAPACITY_UNKNOWN));
				renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (Trailer.MIN_UNKNOWN));
				renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (Trailer.MAX_UNKNOWN));
			end;
			renderText(self.VariableCapacityTrailerHudPosX + 0.015, self.VariableCapacityTrailerHudPosY + 0.03, 0.02, (Trailer.NEW_CAPACITY));
			
			setTextAlignment(RenderText.ALIGN_RIGHT);
			
			renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.15, 0.02, (FillLevelString));
			renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.13, 0.02, (CapacityStartString));
			
			if not self.setNewCapacity then
				if self.fillLevel > (self.capacity * 0.01) then
					if self.capacityTime < 2 then
						renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (NewAddCapacityString));
						renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (AddCapacityMinString));
						renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (AddCapacityMaxString));
					else
						renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (NewRemoveCapacityString));
						renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (RemoveCapacityMinString));
						renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (RemoveCapacityMaxString));
					end;
					renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.03, 0.02, (CapacityString));
				else
					renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (Trailer.UNKNOWN));
					renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (Trailer.UNKNOWN));
					renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (Trailer.UNKNOWN));
					renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.03, 0.02, (Trailer.UNKNOWN));
				end;
			else
				renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.11, 0.02, (Trailer.UNKNOWN));
				renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.07, 0.02, (Trailer.UNKNOWN));
				renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.05, 0.02, (Trailer.UNKNOWN));
				renderText(self.VariableCapacityTrailerHudPosX + 0.4, self.VariableCapacityTrailerHudPosY + 0.03, 0.02, (Trailer.UNKNOWN));
			end;
			
			g_currentMission:addHelpButtonText(Trailer.HIDE_HUD, InputBinding.SHOW_HUD_BUTTON);
		else
			if not self.setNewCapacity then
				if self.fillLevel > (self.capacity * 0.01) then
					if self.capacityTime < 2 then
						g_currentMission:addExtraPrintText(Trailer.CAPACITY_ADDED .. " " .. NewAddCapacityString);
						if self.newAddCapacity > 0 then
							if self.allowAddCapacity then
								g_currentMission:addExtraPrintText(Trailer.ADD_CAPACITY_ACTIVE);
							else
								g_currentMission:addExtraPrintText(Trailer.ADD_CAPACITY_NOT_ACTIVE);
							end;
						else
							g_currentMission:addExtraPrintText(Trailer.ADD_CAPACITY_NOT_ACTIVE);
						end;
					else
						g_currentMission:addExtraPrintText(Trailer.CAPACITY_REMOVED .. " " .. NewRemoveCapacityString);
						if self.newRemoveCapacity > 0 then
							if self.allowAddCapacity then
								g_currentMission:addExtraPrintText(Trailer.REMOVE_CAPACITY_ACTIVE);
							else
								g_currentMission:addExtraPrintText(Trailer.REMOVE_CAPACITY_NOT_ACTIVE);
							end;
						else
							g_currentMission:addExtraPrintText(Trailer.REMOVE_CAPACITY_NOT_ACTIVE);
						end;
					end;
					g_currentMission:addExtraPrintText(Trailer.NEW_CAPACITY .. " " .. CapacityString);
				else
					g_currentMission:addExtraPrintText(Trailer.NEW_CAPACITY .. " " .. CapacityStringUnknown);
				end;
			else
				g_currentMission:addExtraPrintText(Trailer.NEW_CAPACITY .. " " .. CapacityStringUnknown);
			end;
			
			g_currentMission:addHelpButtonText(Trailer.SHOW_HUD, InputBinding.SHOW_HUD_BUTTON);
		end;
	end;
end;